@@ -1741,17 +1741,30 @@ set_palette (TerminalProfile *profile,
 {  
   if (candidate_str != NULL)
     {
+      int i;
       GdkColor new_palette[TERMINAL_PALETTE_SIZE];
 
       if (!terminal_palette_from_string (candidate_str,
                                          new_palette,
                                          TRUE))
-        return FALSE;
-
-      if (memcmp (profile->priv->palette, new_palette,
-                  TERMINAL_PALETTE_SIZE * sizeof (GdkColor)) == 0)
-        return FALSE;
+        {
+          return FALSE;
+        }
 
+      for (i = 0; i < TERMINAL_PALETTE_SIZE; i++)
+        {
+          if (!gdk_color_equal (&profile->priv->palette[i],
+                                &new_palette[i]))
+            {
+              break;
+            }
+        }
+      
+      if (i == TERMINAL_PALETTE_SIZE)
+        {
+          return FALSE;
+        }
+              
       memcpy (profile->priv->palette, new_palette,
               TERMINAL_PALETTE_SIZE * sizeof (GdkColor));
 
@@ -3270,6 +3283,27 @@ terminal_palette_rxvt[TERMINAL_PALETTE_SIZE] =
   { 0, 0xffff, 0xffff, 0xffff }
 };
 
+const GdkColor
+terminal_palette_tango[TERMINAL_PALETTE_SIZE] =
+{
+  { 0, 0x2e2e, 0x3434, 0x3636 },
+  { 0, 0xcccc, 0x0000, 0x0000 },
+  { 0, 0x4e4e, 0x9a9a, 0x0606 },
+  { 0, 0xc4c4, 0xa0a0, 0x0000 },
+  { 0, 0x3434, 0x6565, 0xa4a4 },
+  { 0, 0x7575, 0x5050, 0x7b7b },
+  { 0, 0x0606, 0x9820, 0x9a9a },
+  { 0, 0xd3d3, 0xd7d7, 0xcfcf },
+  { 0, 0x5555, 0x5757, 0x5353 },
+  { 0, 0xefef, 0x2929, 0x2929 },
+  { 0, 0x8a8a, 0xe2e2, 0x3434 },
+  { 0, 0xfcfc, 0xe9e9, 0x4f4f },
+  { 0, 0x7272, 0x9f9f, 0xcfcf },
+  { 0, 0xadad, 0x7f7f, 0xa8a8 },
+  { 0, 0x3434, 0xe2e2, 0xe2e2 },
+  { 0, 0xeeee, 0xeeee, 0xecec }
+};
+
 void
 profile_name_entry_notify (TerminalProfile *profile)
 {
